"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _DB = _interopRequireDefault(require("./DB"));
var _QueryBuilder = require("./QueryBuilder");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Representa un modelos de base de datos
class Model {
  static get repository() {
    throw new Error('database not defined');
  }
  static get tableName() {
    return this.repository.tableName;
  }
  static get datebasaName() {
    return this.repository.databaseName;
  }
  static get primaryKey() {
    return this.repository.primaryKey ?? "id";
  }
  constructor() {
    let props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    this.setProperties(props);
  }
  getProperties() {
    return this;
  }
  setProperties(props) {
    // const cm = this.constructor
    // const campos = Object.getOwnPropertyNames(this);
    // console.debug("__constructor__", cm, campos)

    const instance = this;
    for (var key in props) {
      const value = props[key];
      instance[key] = value;
    }
  }

  /**
   * ```js
   * Animal.executeSql("SELECT * FROM tb_animal WHERE id = ?", [7])
   * ```
   * @return {ResultSet} result
   */
  static async executeSql(sql) {
    let params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    const db = _DB.default.get(this.datebasaName);
    await db.open();
    return await db.executeSql(sql, params);
  }

  /**
   * ```js
   * Animal.all()
   * ```
   * @return array con todos los registros
   */
  static all() {
    const sql = `
      SELECT * FROM ${this.tableName}
    `;
    return this.executeSql(sql).then(result => result.rows);
  }

  /**
   * ```js
   * Animal.findBy("color", "LIKE", '%Brown%')
   * ```
   * @param column mombre de la columna
   * @param op operador: =, <>, <, <=, >, >=, LIKE
   * @param value qriterio de busqueda
   * @return registro
   */
  static findBy(column, op, value) {
    const sql = `
      SELECT * FROM ${this.tableName} WHERE ${column} ${op} ?
    `;
    return this.executeSql(sql, [value]).then(result => result.rows[0]).then(row => row ? new this(row) : row);
  }

  /**
   * ```js
   * Animal.find(1)
   * ```
   * @param id identificador del registro
   * @return registro
   */
  static find(id) {
    return this.findBy(this.primaryKey, "=", id);
  }

  /**
   * ```js
   * Animal.first()
   * ```
   * @return primer registro
   */
  static first() {
    const sql = `
      SELECT * FROM ${this.tableName} ORDER BY ROWID ASC LIMIT 1
    `;
    return this.executeSql(sql).then(result => result.rows[0]).then(row => row ? new this(row) : row);
  }

  /**
   * ```js
   * Animal.last()
   * ```
   * @return último registro
   */
  static last() {
    const sql = `
      SELECT * FROM ${this.tableName} ORDER BY ROWID DESC LIMIT 1
    `;
    return this.executeSql(sql).then(result => result.rows[0]).then(row => row ? new this(row) : row);
  }

  /**
   * ```js
   * Animal.count()
   * ```
   * @return numero de registros
   */
  static count() {
    const sql = `
      SELECT COUNT(*) AS __count__ FROM ${this.tableName}
    `;
    return this.executeSql(sql).then(result => result.rows[0]).then(row => row["__count__"]);
  }

  /**
   * ```js
   * Animal.query({
   *   columns: 'id, name',
   *   where: {
   *     clause: 'age > ? AND age < ?',
   *     args: [ 2, 10 ],
   *   },
   *   page: 2,
   *   limit: 30,
   *   order: 'name ASC'
   * })
   * ```
   * @param {QueryOptions} options
   * @returns array de registros
   */
  static query() {
    var _options$where;
    let options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    const sql = _QueryBuilder.QueryBuilder.query(this.tableName, options);
    return this.executeSql(sql, (_options$where = options.where) === null || _options$where === void 0 ? void 0 : _options$where.args).then(result => result.rows);
  }

  /**
   * ```js
   * Animal.create({
   *   age: 10,
   *   color: '%Brown%',
   * })
   * ```
   * @param obj
   * @returns registro creado
   */
  static create(obj) {
    var sql = _QueryBuilder.QueryBuilder.create(this.tableName, obj);
    const params = Object.values(obj);
    return this.executeSql(sql, params).then(result => this.find(result.insertId));
  }

  /**
   * ```js
   * Animal.update({
   *   id: 100,
   *   age: 10,
   *   color: '%Brown%',
   * })
   * ```
   * @param obj
   * @returns registro actualizado
   */
  static update(obj) {
    // Extrae el valor de "id" y crea un nuevo objeto sin ese dato
    const {
      [this.primaryKey]: id,
      ...props
    } = obj;
    const sql = _QueryBuilder.QueryBuilder.update(this.tableName, props, `${this.primaryKey} = ?`);
    const params = Object.values(props);
    return this.executeSql(sql, [...params, id]).then(_ => this.find(id));
  }

  /**
   * ```js
   * Animal.destroy(100)
   * ```
   * @param id identificador
   * @returns rowsAffected
   */
  static destroy(id) {
    const sql = _QueryBuilder.QueryBuilder.destroy(this.tableName, `${this.primaryKey} = ?`);
    return this.executeSql(sql, [id]).then(res => res.rowsAffected);
  }

  /**
   * ```js
   * Animal.destroyAll()
   * ```
   * @returns rowsAffected
   */
  static destroyAll() {
    const sql = _QueryBuilder.QueryBuilder.destroyAll(this.tableName);
    return this.executeSql(sql).then(res => res.rowsAffected);
  }

  /**
   * ```js
   * Animal.save({
   *   id: 100,
   *   age: 10,
   *   color: '%Brown%',
   * })
   * ```
   * @param obj
   * @returns registro guardo o actualizado
   */
  save() {
    const model = this.constructor;
    const obj = this;
    const id = obj[model.primaryKey];
    if (id) {
      return model.update(obj);
    } else {
      return model.create(obj);
    }
  }
  destroy() {
    const model = this.constructor;
    const obj = this;
    const id = obj[model.primaryKey];
    return model.destroy(id);
  }
}
exports.default = Model;
//# sourceMappingURL=Model.js.map