import type { ItScheme } from "./ItScheme";
import { type QueryOptions, type Where } from "./QueryBuilder";
export type SQLiteDatabase = {
    close(): () => void;
    transaction: (tx: any) => void;
};
/**
 * Resultado de los querys
 */
export type ResultSet = {
    insertId?: number;
    rowsAffected: number;
    rows: any[];
};
/**
 * Administra la base de datos.
 */
export default class DB {
    private static _instances;
    name: string;
    sqlite?: SQLiteDatabase | null;
    /**
     * Abre la base de datos
     *
     * @param {string} name nombre de la db
     */
    constructor(name: string);
    static get(name: string): DB;
    /**
     * Abre la base de datos
     * @returns {SQLiteDatabase}
     */
    open(): Promise<SQLiteDatabase>;
    /**
     * Cierra la db
     */
    close(): Promise<void>;
    /**
     * Ejecuta sentecias sql.
     *
     * @param {string} sql sentencia
     * @param {any[]} params parametros
     *
     * @returns {
     *  "insertId": $insertId,
     *  "rows": [
     *   {"$columnName": $value}
     *  ]
     * }
     */
    executeSql(sql: string, params?: any[]): Promise<ResultSet>;
    /**
     * Obtiene el numero de version de la db
     */
    getVersion(): Promise<number>;
    /**
     * Establece la versión de la base de datos.
     * @param {int} version la nueva versión de la base de datos
     */
    setVersion(version: number): Promise<any>;
    /**
     * Inicializa el scheme de la base de datos.
     */
    init(scheme: ItScheme, version: number): Promise<void>;
    /**
     * SELECT
     *
     * @param {string} table name
     * @param {QueryOptions} options
     * @returns array de registros
     */
    query(table: string, options?: QueryOptions): Promise<any[]>;
    /**
     * INSERT
     *
     * @param {string} table name
     * @param {any} values
     * @returns insertId
     */
    insert(table: string, values: any): Promise<number>;
    /**
     * UPDATE
     *
     * @param {string} table name
     * @param {any} values
     * @param {Where} where
     * @returns rowsAffected
     */
    update(table: string, values: any, where: Where): Promise<number>;
    /**
     * DELETE
     *
     * @param {string} table name
     * @param {Where} where
     * @returns rowsAffected
     */
    delete(table: string, where: Where): Promise<number>;
    /**
     * Compila una clausula where
     *
     * @param {any[]} array [1,3,4,2]
     * @returns (?, ?, ?, ?)
     */
    static whereClausePlaceholders(array: any[]): string;
    /**
     * @param sqlString "valor con 'comillas'"
     * @returns "'valor con \'comillas\''"
     */
    static escapeSqlString(sqlString: string, escape?: string): string;
}
//# sourceMappingURL=DB.d.ts.map